#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <gl/gl.h>
#include <dmedia/vl.h>
 
VLServer svr;
VLPath path;
VLNode src, drn;
VLControlValue val;
VLBuffer capbuffer;
VLInfoPtr info;
char *dataPtr;
int c;
int xsize;
int ysize;
long win;
unsigned char *framebuf;
int i;

void convert(int x, int y, char *inbuf, char *outbuf)
  {
     int i, j;
     long pom;

     /* luminance msbyte copy */
     for (i=0; i<x*y; i++)
	outbuf[i] = inbuf[(2 * i) + 1];

     /* chrominance convertion */
     for (i=0; i < y/2; i++)
	for(j=0; j < x/2; j++) 
	  {
	     pom =    (char)inbuf[(2*i)*2*x + (4*j)] 
		    + (char)inbuf[(2*i + 1)*2*y + (4*j)];
	     pom = pom/2;
	     outbuf[x*y + i*x/2 + j] = (char)pom;

	     pom =    (char)inbuf[(2*i)*2*x + (4*j) + 2] 
		    + (char)inbuf[(2*i + 1)*2*x + (4*j) + 2];
	     pom = pom/2;
	     outbuf[x*y + x*y/4 + i*x/2 + j] = (char)pom;
	  }
     
     return;
  }

/* Report errors */
void
error_exit(void)
{
    vlPerror("");
    exit(1);
}

void
initcapture(int x, int y)
{

    foreground();
        
    /* Connect to the daemon */
    if (!(svr = vlOpenVideo(""))) 
	error_exit();

    /* Set up a drain node in memory */
    drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
    
    /* Set up a source node on any video source  */
    src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);

    /* Create a path using the first device that will support it */
    path = vlCreatePath(svr, VL_ANY, src, drn); 

    /* Set up the hardware for and define the usage of the path */
    if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
	error_exit();

    val.intVal = 5;
    vlSetControl(svr, path, drn, VL_RATE, &val);

    val.fractVal.numerator = 1;
    val.fractVal.denominator = 3;
    vlSetControl(svr, path, drn, VL_ZOOM, &val);

    val.xyVal.x = x;
    val.xyVal.y = y;
    vlSetControl(svr, path, drn, VL_SIZE, &val);

    /* Set the packing to YUV 4:2:2 */
    val.intVal = VL_PACKING_YVYU_422_8;
    vlSetControl(svr, path, drn, VL_PACKING, &val);
    
    /* Get the video size */
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    xsize = val.xyVal.x;
    ysize = val.xyVal.y;
    
    /* Set up and open a GL window to display the data */
/*    prefsize(xsize,ysize);
    win = winopen("Simpleccapt Window");
    RGBmode();
    pixmode(PM_TTOB, 1);
    gconfig();
*/    
    /* Create and register a buffer for 1 frame */
    capbuffer = vlCreateBuffer(svr, path, drn, 1);
    if (capbuffer == NULL)
	error_exit();	
    vlRegisterBuffer(svr, path, drn, capbuffer);
    
    /* Begin the data transfer */
    if (vlBeginTransfer(svr, path, 0, NULL))
	error_exit();
    
    return;    
}

unsigned char *getFrame(int x, int y)
{
  unsigned char *buf;

  buf=(unsigned char *)malloc(38016);

  do {
	    sginap(1);		/* wait a tick */
	    info = vlGetNextValid(svr, capbuffer);
	} while (!info);
    
	/* Get a pointer to the frame */
	dataPtr = vlGetActiveRegion(svr, capbuffer, info);
		
	/* Write the data to the screen */
/*	lrectwrite(0,0, xsize-1, ysize-1, (ulong *)dataPtr);
*/
	convert(x, y, dataPtr, buf);
	
	/* Finished with frame, unlock the buffer */
	vlPutFree(svr, capbuffer);

  return buf;
}

closecapture()
{
    /* End the data transfer */
    vlEndTransfer(svr, path);
     
    /* Cleanup before exiting */
    vlDeregisterBuffer(svr, path, drn, capbuffer);
    vlDestroyBuffer(svr, capbuffer);
    vlDestroyPath(svr, path);
    vlCloseVideo(svr);
}






